from pathlib import Path
from .TypedPath import TypedPath
from .TimestampedPath import TimestampedPath

class IClearIO:
	"""
	Interface class for all classes that wish to represent themselves as ClearIO compliant classes.
	"""

	def CreateDirectory(self, fullDirectoryPath:Path|str, createParents:bool = True) -> None:
		"""
		Creates a directory

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to create.

		createParents : bool
			Indicates whether to create the parent directories. If False and parent directories are missing, an IOError is raised.
		"""
		raise NotImplementedError()

	def RemoveDirectory(self, fullDirectoryPath:Path|str) -> None:
		"""
		Removes the specified directory and all child objects within the directory.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to remove.
		"""
		raise NotImplementedError()

	def EmptyDirectory(self, fullDirectoryPath:Path|str) -> None:
		"""
		Removes all child objects within the directory, but does not remove the directory.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to clean.
		"""
		raise NotImplementedError()

	def ListAllDirectories(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		"""
		Retrieves all directory paths within a directory. This method is not recursive.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		Returns
		-------
		list[Path or str]
			A list of directories within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListAllDirectoriesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		"""
		Recursively retrieves all directory paths within a directory.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		Returns
		-------
		list[Path or str]
			A list of directories within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListDirectories(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		"""
		Retrieves directory paths within a directory where the directory name matches a regex pattern. This method is not recursive.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		pattern : str
			The pattern to use when selecting which files to include in the results.
			Note: This is regex based. Not based on typical file name wildcards.
			Cf. https://medium.com/@jamestjw/parsing-file-names-using-regular-expressions-3e85d64deb69

		Returns
		-------
		list[Path or str]
			A list of directories within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListDirectoriesRecursively(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		"""
		Recursively retrieves directory paths within a directory where the directory name matches a regex pattern.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		pattern : str
			The pattern to use when selecting which files to include in the results.
			Note: This is regex based. Not based on typical file name wildcards.
			Cf. https://medium.com/@jamestjw/parsing-file-names-using-regular-expressions-3e85d64deb69

		Returns
		-------
		list[Path or str]
			A list of directories within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListAllFiles(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		"""
		Retrieves all file paths within a directory. This method is not recursive.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		Returns
		-------
		list[Path or str]
			A list of files within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListAllFilesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		"""
		Recursively retrieves all file paths within a directory.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		Returns
		-------
		list[Path or str]
			A list of files within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListFiles(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		"""
		Retrieves file paths within a directory where the file name matches a regex pattern. This method is not recursive.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		pattern : str
			The pattern to use when selecting which files to include in the results.
			Note: This is regex based. Not based on typical file name wildcards.
			Cf. https://medium.com/@jamestjw/parsing-file-names-using-regular-expressions-3e85d64deb69

		Returns
		-------
		list[Path or str]
			A list of files within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListFilesRecursively(self, fullDirectoryPath:TypedPath|Path|str, pattern:str|None = None) -> list[TypedPath]:
		"""
		Recursively retrieves file paths within a directory where the file name matches a regex pattern.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		pattern : str
			The pattern to use when selecting which files to include in the results.
			Note: This is regex based. Not based on typical file name wildcards.
			Cf. https://medium.com/@jamestjw/parsing-file-names-using-regular-expressions-3e85d64deb69

		Returns
		-------
		list[Path or str]
			A list of files within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def List(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		"""
		Retrieves all files and directories paths within a directory.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		Returns
		-------
		list[Path or str]
			A list of files and directories within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def ListRecursively(self, fullDirectoryPath:TypedPath|Path|str) -> list[TypedPath]:
		"""
		Recursively retrieves all files and directories paths within a directory.

		Parameters
		----------
		fullDirectoryPath : Path or str
			The full path to the directory to retrieve.

		Returns
		-------
		list[Path or str]
			A list of files and directories within the directory.
			Return type is determined by the type of fullDirectoryPath.
		"""
		raise NotImplementedError()

	def GetFile(self, fullPath:Path|str) -> bytes:
		"""
		Retrieves the bytes of a file.
		Note: These bytes can be used in PutFile to perform a copy style of operation.

		Parameters
		----------
		fullPath : Path or str
			The full path to the file to retrieve.
			
		Returns
		-------
		bytes
			A byte array of the contents of the file.
		"""
		raise NotImplementedError()

	def PutFile(self, fullPath:Path|str, content:bytes):
		"""
		Writes a byte array to a file.
		Note: contents may be the output of GetFile to perform a copy style of operation.

		Parameters
		----------
		fullPath : Path or str
			The full path to the file to write.

		contents : bytes
			The array of bytes to write to the file.
		"""
		raise NotImplementedError()

	def RemoveFile(self, fullFilePath:Path|str) -> None:
		"""
		Removes the specified file.

		Parameters
		----------
		fullFilePath : Path or str
			The full path to the file to remove.
		"""
		raise NotImplementedError()

	def ParseTimestamps(self, paths:list[TypedPath], prefixesBeforeTimestamp:list[str] | None = None, timestampFormat:str | None = None) -> list[TimestampedPath]:
		"""
		Attempts to parse timestamps from the file/directory names in the list provided.

		Parameters
		----------
		paths : list[Path or str]
			A list of paths to attempt to parse timestamps from. Can be directories, file, or a combination of both.
		
		prefixesBeforeTimestamp : list[str]
			A list of prefixes to check before the timestamp. If the list contains "" or is None, if will attempt to treat the stem, file name without extension, as the timestamp.
		
		timestampFormat : str
			The format the timestamp is expected to be in.
			Cf. https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes

		Returns
		-------
		list[TimestampedPath]
			A list containing TimestampedPaths. If a timestamp cannot be parsed, the TimestampedPath.HasTimestamp will be False.
		"""
		raise NotImplementedError()

__all__ = ["IClearIO"]



from enum import Enum
from datetime import datetime
from pathlib import PureWindowsPath
import pathlib

class IOObjectType(Enum):
	Unknown = 0
	File = 1
	Directory = 2

class TypedPath(type(pathlib.Path())):
	ObjectType:IOObjectType = IOObjectType.Unknown
	SystemSeparator:str = "/"

	def __init__(self, path:pathlib.Path|str, objectType:IOObjectType = IOObjectType.Unknown):
		if (isinstance(path, PureWindowsPath)):
			self.SystemSeparator = "\\"
		if (isinstance(path, str)):
			super().__init__(pathlib.Path(path))
		else:
			super().__init__(path)
		self.Type = objectType

	def __str__(self):
		return str(super().__str__()).replace("\\", self.SystemSeparator)

__all__ = ["IOObjectType", "TypedPath"]



from datetime import datetime
import pathlib
from .TypedPath import TypedPath

class TimestampedPath(TypedPath):
	Timestamp:datetime|None = None
	TimestampString:str|None = None
	HasTimestamp:bool = False	

	def __init__(self, path:TypedPath, prefixesBeforeTimestamp:list[str] | None = None, timestampFormat:str | None = None):
		if (isinstance(path, pathlib.Path)):
			super().__init__(str(path))
		else:
			super().__init__(path)
		try:
			timestampLength:int = len(datetime.now().strftime(timestampFormat))
			fileName:str = str(super().stem)
			if (prefixesBeforeTimestamp is not None and len(prefixesBeforeTimestamp) > 0):
				for prefix in prefixesBeforeTimestamp:
					if (prefix == "" or prefix is None):
						if (len(fileName) == timestampLength):
							self.TimestampString = fileName
							self.Timestamp = datetime.strptime(self.TimestampString, timestampFormat)
					elif (fileName.startswith(prefix)):
						timestampString:str = str(fileName)[len(prefix):len(prefix)+timestampLength]
						self.TimestampString = timestampString
						self.Timestamp = datetime.strptime(self.TimestampString, timestampFormat)
						break
			else:
				if (len(fileName) == timestampLength):
					self.TimestampString = fileName
					self.Timestamp = datetime.strptime(self.TimestampString, timestampFormat)
			if (self.Timestamp is None):
				self.TimestampString = None
				self.HasTimestamp = False
			else:
				self.HasTimestamp = True
		except:
			self.HasTimestamp = False	

	def __str__(self):
		return str(super().__str__())

__all__ = ["TimestampedPath"]



from enum import Enum
from pathlib import Path
import platform
import re
from .IClearIO import IClearIO
from .TypedPath import TypedPath, IOObjectType
from .TimestampedPath import TimestampedPath

if (platform.system() == "Linux"):
	import smbclient
	import smbclient.shutil
if (platform.system() == "Windows"):
	import shutil
	import pywintypes
	import win32wnet

class WindowsSMBClearIO(IClearIO):
	HostName:str|None = None
	ShareName:str|None = None
	FullName:str|None = None

	def __init__(self, hostName:str, shareName:str, userName:str|None = None, password:str|None = None) -> None:
		super().__init__()
		self.HostName = hostName
		self.ShareName = shareName
		self.FullName = f"\\\\{hostName}\\{shareName}"
		netResource = win32wnet.NETRESOURCE()
		netResource.lpRemoteName = self.FullName
		win32wnet.WNetAddConnection2(netResource, password, userName, 0)

	def CreateDirectory(self, fullDirectoryPath:Path|str, createParents:bool = True) -> None:
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		fullDirectoryPath.mkdir(parents=createParents, exist_ok=True)

	def RemoveDirectory(self, fullDirectoryPath:Path|str) -> None:
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.iterdir():
			if item.is_dir():
				self.RemoveDirectory(item)
			else:
				item.unlink()
		fullDirectoryPath.rmdir()

	def EmptyDirectory(self, fullDirectoryPath:Path|str) -> None:
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.iterdir():
			if item.is_dir():
				self.RemoveDirectory(item)
			else:
				item.unlink()

	def ListAllDirectories(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.iterdir():
			if (item.is_dir()):
				returnValue.append(TypedPath(item, IOObjectType.Directory))
		return returnValue

	def ListAllDirectoriesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.rglob("*"):
			if (item.is_dir()):
				returnValue.append(TypedPath(item, IOObjectType.Directory))
		return returnValue

	def ListDirectories(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		allItems:list[Path] = self.ListAllDirectories(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, str(item.stem))):
					returnValue.append(item)
		return returnValue

	def ListDirectoriesRecursively(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		allItems:list[Path] = self.ListAllDirectoriesRecursively(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, str(item.stem))):
					returnValue.append(item)
		return returnValue

	def ListAllFiles(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.iterdir():
			if (item.is_file()):
				returnValue.append(TypedPath(item, IOObjectType.File))
		return returnValue

	def ListAllFilesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.rglob("*"):
			if (item.is_file()):
				returnValue.append(TypedPath(item, IOObjectType.File))
		return returnValue

	def ListFiles(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		allItems:list[Path] = self.ListAllFiles(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, str(item.stem))):
					returnValue.append(item)
		return returnValue

	def ListFilesRecursively(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		allItems:list[Path] = self.ListAllFilesRecursively(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, str(item.stem))):
					returnValue.append(item)
		return returnValue

	def List(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.glob("*"):
			returnValue.append(TypedPath(item, IOObjectType.File if item.is_file() else IOObjectType.Directory))
		return returnValue

	def ListRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, str)):
			fullDirectoryPath = Path(fullDirectoryPath)
		for item in fullDirectoryPath.rglob("*"):
			returnValue.append(TypedPath(item, IOObjectType.File if item.is_file() else IOObjectType.Directory))
		return returnValue

	def GetFile(self, fullPath:Path|str) -> bytes:
		if (isinstance(fullPath, str)):
			fullPath = Path(fullPath)
		return fullPath.read_bytes()

	def PutFile(self, fullPath:Path|str, contents:bytes):
		if (isinstance(fullPath, str)):
			fullPath = Path(fullPath)
		fullPath.write_bytes(contents)

	def RemoveFile(self, fullFilePath:Path|str) -> None:
		if (isinstance(fullFilePath, str)):
			fullFilePath = Path(fullFilePath)
		if (fullFilePath.is_file):
			fullFilePath.unlink(True)

	def ParseTimestamps(self, paths:list[TypedPath], prefixesBeforeTimestamp:list[str] | None = None, timestampFormat:str | None = None) -> list[TimestampedPath]:
		returnValue:list[TimestampedPath] = list[TimestampedPath]()
		for item in paths:
			returnValue.append(TimestampedPath(item, prefixesBeforeTimestamp, timestampFormat))
		return returnValue

class LinuxSMBClearIO(IClearIO):
	HostName:str|None = None
	ShareName:str|None = None
	FullName:str|None = None

	def __init__(self, hostName:str, shareName:str, userName:str|None = None, password:str|None = None) -> None:
		super().__init__()
		self.HostName = hostName
		self.ShareName = shareName
		self.FullName = f"\\\\{hostName}\\{shareName}"
		smbclient.register_session(self.HostName, username=userName, password=password)

	def CreateDirectory(self, fullDirectoryPath:Path|str, createParents:bool = True) -> None:
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		lastSeparator:int = fullDirectoryPath.rfind("\\")
		if (not smbclient.path.exists(fullDirectoryPath)):
			if (smbclient.path.exists(fullDirectoryPath[:lastSeparator])):
				smbclient.mkdir(fullDirectoryPath)
			elif (createParents):
				smbclient.makedirs(fullDirectoryPath, True)
			else:
				raise FileNotFoundError("Parent directory not found")

	def RemoveDirectory(self, fullDirectoryPath:Path|str) -> None:
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for item in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{item}"
			if (smbclient.path.isdir(fullPath)):
				self.RemoveDirectory(fullPath)
			else:
				smbclient.remove(fullPath)
		smbclient.rmdir(fullDirectoryPath)

	def EmptyDirectory(self, fullDirectoryPath:Path|str) -> None:
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for item in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{item}"
			if (smbclient.path.isdir(fullPath)):
				self.RemoveDirectory(fullPath)

	def ListAllDirectories(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[Path|str] = list()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for item in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{item}"
			if (smbclient.path.isdir(fullPath)):
				returnValue.append(TypedPath(fullPath, IOObjectType.Directory))
		return returnValue

	def ListAllDirectoriesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[Path|str] = list()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for item in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{item}"
			if (smbclient.path.isdir(fullPath)):
				returnValue.append(TypedPath(fullPath, IOObjectType.Directory))
				returnValue += self.ListAllDirectoriesRecursively(fullPath)
		return returnValue

	def ListDirectories(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		allItems:list[Path] = self.ListAllDirectories(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, Path(item).stem)):
					returnValue.append(item)
		return returnValue

	def ListDirectoriesRecursively(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		allItems:list[Path] = self.ListAllDirectoriesRecursively(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, Path(item).stem)):
					returnValue.append(item)
		return returnValue

	def ListAllFiles(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[Path|str] = list()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for file in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{file}"
			if (smbclient.path.isfile(fullPath)):
				returnValue.append(TypedPath(fullPath, IOObjectType.File))
		return returnValue

	def ListAllFilesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[Path|str] = list()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		directories:list[TypedPath] = self.ListDirectoriesRecursively(fullDirectoryPath)
		directories.append(fullDirectoryPath)
		for directory in directories:
			for file in smbclient.listdir(directory):
				fullPath:str = f"{directory}\\{file}"
				if (smbclient.path.isfile(fullPath)):
					returnValue.append(TypedPath(fullPath, IOObjectType.File))
		return returnValue

	def ListFiles(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		allItems:list[Path] = self.ListAllFiles(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, Path(item).stem)):
					returnValue.append(item)
		return returnValue

	def ListFilesRecursively(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		returnValue:list[TypedPath] = list[TypedPath]()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		allItems:list[Path] = self.ListAllFilesRecursively(fullDirectoryPath)
		if (pattern == "*.*" or pattern == "*" or pattern == "" or pattern is None):
			for item in allItems:
				returnValue.append(item)
		else:
			for item in allItems:
				if (re.search(pattern, Path(item).stem)):
					returnValue.append(item)
		return returnValue

	def List(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[Path|str] = list()
		found:list[str] = list()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for item in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{item}"
			returnValue.append(TypedPath(fullPath, IOObjectType.File if smbclient.path.isfile(fullPath) else IOObjectType.Directory))
		return returnValue

	def ListRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		returnValue:list[Path|str] = list()
		found:list[str] = list()
		outputType:str = "Path" if (isinstance(fullDirectoryPath, Path)) else "str"
		if (isinstance(fullDirectoryPath, Path)):
			fullDirectoryPath = str(fullDirectoryPath)
		for item in smbclient.listdir(fullDirectoryPath):
			fullPath:str = f"{fullDirectoryPath}\\{item}"
			returnValue.append(TypedPath(fullPath, IOObjectType.File if smbclient.path.isfile(fullPath) else IOObjectType.Directory))
			if (smbclient.path.isdir(fullPath)):
				returnValue += self.ListRecursively(fullPath)
		return returnValue

	def GetFile(self, fullPath:Path|str) -> bytes:
		returnValue:bytes = bytes()
		if (isinstance(fullPath, Path)):
			fullPath = str(fullPath)
			with smbclient.open_file(fullPath, "r") as file:
				returnValue = file.read()
		return returnValue

	def PutFile(self, fullPath:Path|str, contents:bytes):
		if (isinstance(fullPath, Path)):
			fullPath = str(fullPath)
			with smbclient.open_file(fullPath, "w") as file:
				file.write(contents)

	def RemoveFile(self, fullFilePath:Path|str) -> None:
		if (isinstance(fullFilePath, Path)):
			fullFilePath = str(fullFilePath)
		if (smbclient.path.isfile(fullFilePath)):
			smbclient.remove(fullFilePath)

	def ParseTimestamps(self, paths:list[TypedPath], prefixesBeforeTimestamp:list[str] | None = None, timestampFormat:str | None = None) -> list[TimestampedPath]:
		returnValue:list[TimestampedPath] = list[TimestampedPath]()
		for item in paths:
			returnValue.append(TimestampedPath(item, prefixesBeforeTimestamp, timestampFormat))
		return returnValue

class SMBClearIO(IClearIO):
	"""
	Wrapper class for SMB access in a ClearIO compliant way.
	This class wraps SMB classes for either Windows or Linux SMB depending on which system it is ran on.
	"""

	smbClearIO:IClearIO|None = IClearIO()
	System:str = platform.system()

	def __init__(self, hostName:str, shareName:str, userName:str|None = None, password:str|None = None) -> None:
		"""
		Wrapper class for SMB access in a ClearIO compliant way.
		This class wraps SMB classes for either Windows or Linux SMB depending on which system it is ran on.
		

		Parameters
		----------
		hostName : str
			The name of the SMB (Windows or Samba on Linux) server.

		shareName : str
			The name of the SMB (Windows or Samba on Linux) share on the server.

		userName : str
			The user to connect to the SMB resource with.

		password : str
			The password for the user to connect to the SMB resource with.
		"""
		if (self.System == "Linux"):
			self.smbClearIO = LinuxSMBClearIO(hostName, shareName, userName, password)
		if (self.System == "Windows"):
			self.smbClearIO = WindowsSMBClearIO(hostName, shareName, userName, password)

	def CreateDirectory(self, fullDirectoryPath:Path|str, createParents:bool = True) -> None:
		self.smbClearIO.CreateDirectory(fullDirectoryPath, createParents)

	def RemoveDirectory(self, fullDirectoryPath:Path|str) -> None:
		self.smbClearIO.RemoveDirectory(fullDirectoryPath)

	def EmptyDirectory(self, fullDirectoryPath:Path|str) -> None:
		self.smbClearIO.EmptyDirectory(fullDirectoryPath)

	def ListDirectories(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListDirectories(fullDirectoryPath))

	def ListDirectoriesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListDirectoriesRecursively(fullDirectoryPath))

	def ListAllFiles(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListAllFiles(fullDirectoryPath))

	def ListAllFilesRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListAllFilesRecursively(fullDirectoryPath))

	def ListFiles(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListFiles(fullDirectoryPath, pattern))

	def ListFilesRecursively(self, fullDirectoryPath:Path|str, pattern:str|None = None) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListFilesRecursively(fullDirectoryPath, pattern))

	def List(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		return sorted(self.smbClearIO.List(fullDirectoryPath))

	def ListRecursively(self, fullDirectoryPath:Path|str) -> list[TypedPath]:
		return sorted(self.smbClearIO.ListRecursively(fullDirectoryPath))

	def GetFile(self, fullPath:Path|str) -> bytes:
		return self.smbClearIO.GetFile(fullPath)

	def PutFile(self, fullPath:Path|str, content:bytes):
		self.smbClearIO.PutFile(fullPath, content)

	def RemoveFile(self, fullFilePath:Path|str) -> None:
		self.smbClearIO.RemoveFile(fullFilePath)

	def ParseTimestamps(self, paths:list[TypedPath], prefixesBeforeTimestamp:list[str] | None = None, timestampFormat:str | None = None) -> list[TimestampedPath]:
		return sorted(self.smbClearIO.ParseTimestamps(paths, prefixesBeforeTimestamp, timestampFormat))

__all__ = ["WindowsSMBClearIO", "LinuxSMBClearIO", "SMBClearIO"]
